package client

import (
	"context"
	"crypto/tls"
	"crypto/x509"
	"encoding/json"
	"errors"
	"github.com/zhangyuanCloud/common"
	"github.com/zhangyuanCloud/common/utils"
	"gitlab.novgate.com/common/system-client/pb"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials"
	"os"
	"strconv"
	"time"
)

type GrpcClient struct {
	ApiUrl string
	AppKey string
	aes    *utils.AES
	conn   *grpc.ClientConn
	client pb.UserServiceClient
}

// NewGrpcClient 创建一个新的gRPC客户端
func NewGrpcClient(apiUrl, appKey, appSecret string) (*GrpcClient, error) {
	var opts []grpc.DialOption
	opts = append(opts, grpc.WithInsecure())

	proxyCA := "/Users/zhangyuan/Downloads/xmtest.in.pem" // CA cert that signed the proxy
	f, err := os.ReadFile(proxyCA)
	if err != nil {
		panic(err)
	}

	p := x509.NewCertPool()
	p.AppendCertsFromPEM(f)
	tlsConfig := &tls.Config{
		RootCAs: p,
	}
	opts = append(opts, grpc.WithTransportCredentials(credentials.NewTLS(tlsConfig)))
	conn, err := grpc.NewClient(apiUrl, opts...)
	// 如果需要TLS连接，可以配置TLS选项

	//conn, err := grpc.Dial(apiUrl, opts...)
	if err != nil {
		return nil, err
	}

	return &GrpcClient{
		ApiUrl: apiUrl,
		AppKey: appKey,
		aes:    utils.NewAES([]byte(appKey), []byte(appSecret)),
		conn:   conn,
		client: pb.NewUserServiceClient(conn),
	}, nil
}

// Close 关闭gRPC连接
func (c *GrpcClient) Close() error {
	return c.conn.Close()
}

// SayHello 调用SayHello RPC方法
func (c *GrpcClient) Login(name, password string) (string, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
	defer cancel()

	req := &pb.LoginReq{
		Header: &pb.RpcHeader{
			Uuid:  "33333333",
			Token: "",
		},
		UserName: name,
		Password: password,
	}

	rpcParam := c.Encrypt(req)
	resp, err := c.client.Login(ctx, rpcParam)
	if err != nil {
		return "", err
	}

	if resp.Code != strconv.Itoa(common.Success) {
		return "", errors.New(resp.Msg)
	}
	return c.Decrypt([]byte(resp.Data)), nil
}

func (c *GrpcClient) Logout(token string) (string, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
	defer cancel()

	req := &pb.RpcHeader{
		Uuid:  "33333333",
		Token: token,
	}
	rpcParam := c.Encrypt(req)
	resp, err := c.client.Logout(ctx, rpcParam)
	if err != nil {
		return "", err
	}
	if resp.Code != strconv.Itoa(common.Success) {
		return "", errors.New(resp.Msg)
	}
	return c.Decrypt([]byte(resp.Data)), nil
}

func (c *GrpcClient) CheckToken(token string) (string, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
	defer cancel()

	req := &pb.RpcHeader{
		Uuid:  "33333333",
		Token: token,
	}
	rpcParam := c.Encrypt(req)
	resp, err := c.client.CheckToken(ctx, rpcParam)
	if err != nil {
		return "", err
	}
	if resp.Code != strconv.Itoa(common.Success) {
		return "", errors.New(resp.Msg)
	}
	return c.Decrypt([]byte(resp.Data)), nil
}

func (c *GrpcClient) GetUserMenu(token string) (string, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
	defer cancel()

	req := &pb.RpcHeader{
		Uuid:  "33333333",
		Token: token,
	}
	rpcParam := c.Encrypt(req)
	resp, err := c.client.GetUserMenu(ctx, rpcParam)
	if err != nil {
		return "", err
	}
	if resp.Code != strconv.Itoa(common.Success) {
		return "", errors.New(resp.Msg)
	}
	return c.Decrypt([]byte(resp.Data)), nil
}

func (c *GrpcClient) Decrypt(body []byte) string {
	decrypt, err := c.aes.Decrypt(body)
	if err != nil {
		return ""
	}

	return string(decrypt)
}

func (c *GrpcClient) Encrypt(param interface{}) *pb.RpcParam {

	result := &pb.RpcParam{
		AppKey: c.AppKey,
	}

	marshal, _ := json.Marshal(param)

	encrypt, err := c.aes.Encrypt(marshal)
	if err != nil {
		return nil
	}
	result.Param = encrypt
	return result
}
