package client

import (
	"bytes"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/zhangyuanCloud/common"
	"github.com/zhangyuanCloud/common/utils"
	"gitlab.novgate.com/common/system-client/pb"
	"io"
	"net/http"
)

type HttpClient struct {
	ApiUrl string
	appKey string
	client *http.Client
	aes    *utils.AES
}

type RequestParam struct {
	AppKey string `json:"appKey"`
	Param  string `json:"param"`
}

const (
	LoginPath      = "/admin/api/login"
	LogoutPath     = "/admin/api/logout"
	CheckTokenPath = "/admin/api/checkToken"
	UserMenuPath   = "/admin/api/getUserMenu"
)

func NewHttpClient(apiUrl, appKey, appSecret string) *HttpClient {
	return &HttpClient{
		ApiUrl: apiUrl,
		appKey: appKey,
		aes:    utils.NewAES([]byte(appKey), []byte(appSecret)),
		client: &http.Client{
			Timeout: 15,
		},
	}
}
func (c *HttpClient) Login(param *pb.LoginReq) *pb.AdminUser {

	resp := &common.DataResponse{}
	data := &pb.AdminUser{}
	resp.Data = data
	c.doRequest(LoginPath, http.MethodPost, param, resp)

	return data
}
func (c *HttpClient) Logout(token string) *common.DataResponse {
	resp := &common.DataResponse{}
	req := &pb.RpcHeader{Token: token}
	c.doRequest(LogoutPath, http.MethodPost, req, resp)

	return resp
}

func (c *HttpClient) CheckToken(token string) *pb.AdminUser {
	resp := &common.DataResponse{}
	data := &pb.AdminUser{}
	resp.Data = data
	req := &pb.RpcHeader{Token: token}
	c.doRequest(CheckTokenPath, http.MethodPost, req, resp)

	return data
}

func (c *HttpClient) GetUserMenu(token string) []*pb.MenuTreeModel {
	var data []*pb.MenuTreeModel
	resp := &common.DataResponse{
		Data: data,
	}

	req := &pb.RpcHeader{Token: token}
	c.doRequest(UserMenuPath, http.MethodPost, req, resp)

	return data
}

func (c *HttpClient) doRequest(path, method string, params interface{}, result interface{}) error {
	url := c.ApiUrl + path

	requestParam := c.Encrypt(params)
	reqParam, _ := json.Marshal(requestParam)

	req, err := http.NewRequest(method, url, bytes.NewBuffer(reqParam))
	if err != nil {
		return err
	}

	req.Header.Set("Content-Type", "application/json")
	resp, err := c.client.Do(req)
	if err != nil {
		fmt.Println("pay center http request timeout")
		return err
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		msg := fmt.Sprintf("network error: (%d)", resp.StatusCode)
		fmt.Println(msg)
		return errors.New(msg)
	}
	bodyByte, err := io.ReadAll(resp.Body)
	if err != nil {
		fmt.Println("response body read error")
		return errors.New("response body read error")
	}

	fmt.Println("解码前响应数据：", string(bodyByte))
	decrypt := c.Decrypt(bodyByte)

	fmt.Println("解码后响应数据：", decrypt)
	err = json.Unmarshal([]byte(decrypt), result)
	if err != nil {
		fmt.Println("response body unmarshal error")
		return err
	}
	return nil
}

func (c *HttpClient) Decrypt(body []byte) string {
	decrypt, err := c.aes.Decrypt(body)
	if err != nil {
		return ""
	}

	return string(decrypt)
}

func (c *HttpClient) Encrypt(param interface{}) *pb.RpcParam {

	result := &pb.RpcParam{
		AppKey: c.appKey,
	}

	marshal, _ := json.Marshal(param)

	encrypt, err := c.aes.Encrypt(marshal)
	if err != nil {
		return nil
	}
	result.Param = encrypt
	return result
}
